@@ -1,12 +1,13 @@
 package OpenInteract::ErrorObject;
 
-# $Id: ErrorObject.pm,v 1.3 2001/07/13 14:48:51 lachoy Exp $
+# $Id: ErrorObject.pm,v 1.4 2001/09/06 14:34:54 lachoy Exp $
 
 use strict;
 use Data::Dumper   qw( Dumper );
+use SPOPS::Utility;
 use Text::Wrap     ();
 
-$OpenInteract::ErrorObject::VERSION = sprintf("%d.%02d", q$Revision: 1.3 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::ErrorObject::VERSION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);
 
 my @LIST = (); # keeps errors during request
 my $DEFAULT_ACTION = 'n/a';
@@ -26,7 +27,7 @@ sub report        { return \@LIST }
 sub throw {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-  
+
     unless ( $p->{package} and $p->{filename} and $p->{line} ) {
         my ( $cpkg, $cfile, $cline ) = caller;
         $p->{package}  = $cpkg;
@@ -50,14 +51,14 @@ sub initialize {
     $R->DEBUG && $R->scrib( 2, "Initializing new error: ", Dumper( $p ) );
 
     # Set pkg (etc) context if not already set
-  
+
     unless ( $p->{package} and $p->{filename} and $p->{line} ) {
         my ( $cpkg, $cfile, $cline ) = caller;
         $p->{package}  = $cpkg;
         $p->{filename} = $cfile;
-        $p->{line}     = $cline;   
+        $p->{line}     = $cline;
     }
-  
+
     # Set any extra items
 
     my $ei = OpenInteract::Error->get;
@@ -68,7 +69,7 @@ sub initialize {
         $self->{ "tmp_$key" } = $p->{extra}->{ $key };
         $extra_info .= $p->{extra}->{ $key } . "<br>\n";
     }
-  
+
     # If there was nothing passed in except for the code,
     # then get the error information from the package
     # error holding variables.
@@ -77,12 +78,12 @@ sub initialize {
         $p->{ $field } ||= $ei->{ $field };
     }
     $p->{notes} .= $extra_info;
-  
+
     # In any case, clear out any old messages; they're not 
     # needed anymore
 
     OpenInteract::Error->clear;
-  
+
     # Get the action from $R if not already set
 
     $p->{action} ||= $R->{current_context}->{action} || $DEFAULT_ACTION;
@@ -115,7 +116,8 @@ sub initialize {
     $self->{user_id}     = $R->{auth}->{user}->id       if ( ref $R->{auth}->{user} );
     $self->{session_id}  = $R->{session}->{_session_id} if ( ref $R->{session}   );
     $self->{browser}     = $head_in->{'User-Agent'};
-    $self->{error_time}  = $self->now;
+    $self->{error_time}  = SPOPS::Utility->now;
+    $self->{url}         = $R->{path}{original};
     $self->{referer}     = $head_in->{'Referer'};
     return $self;
 }
@@ -125,7 +127,7 @@ sub fail_save {
     my ( $self, $p ) = @_;
     my $R = OpenInteract::Request->instance;
     my $error_dir = $R->CONFIG->get_dir( 'error' );
-  
+
     # Note that even though it's not saved, the ID should still
     # have been defined in the save() process since we
     # are using random codes. Also, do not do a ->throw with any 
@@ -209,20 +211,25 @@ B<throw( \%params )>
 Throws an error from anywhere in the system. We create an error object
 with the parameters (or with the error information already stored, see
 below) and send the object onto the error handling framework within
-OpenInteract. 
+OpenInteract.
 
 Parameters:
 
- code ($)
-   Mandatory. See L<OpenInteract::Error::Main> for what the different
-   codes signify. (Basically, the higher the code the less severe the
-   error.)
+=over 4
+
+B<code> ($)
+
+Mandatory. See L<OpenInteract::Error::Main> for what the different
+codes signify. (Basically, the higher the code the less severe the
+error.)
 
 If you call throw() with only the I<code> parameter, the method will
 snatch the error information from the package variables already
 set. Otherwise, see L<OpenInteract::Error> for the different
 parameters that can be set.
 
+=back
+
 =head1 TO DO
 
 =head1 BUGS
