@@ -1,6 +1,6 @@
 package OpenInteract::FullText;
 
-# $Id: FullText.pm,v 1.8 2001/07/12 23:06:55 lachoy Exp $
+# $Id: FullText.pm,v 1.9 2001/10/01 17:13:51 lachoy Exp $
 
 use strict;
 use Lingua::Stem ();
@@ -11,7 +11,7 @@ BEGIN {
     eval { require OpenInteract::FullTextIterator };
 }
 
-$OpenInteract::FullText::VERSION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::FullText::VERSION = sprintf("%d.%02d", q$Revision: 1.9 $ =~ /(\d+)\.(\d+)/);
 @OpenInteract::FullText::ISA     = ();
 
 use constant COLUMN_GROUP    => 'listing';
@@ -73,16 +73,16 @@ sub search_fulltext_index {
 
     my $R = OpenInteract::Request->instance;
     $R->DEBUG && $R->scrib( 1, "Searching for terms: ", join( ',', @{ $p->{search_terms} } ) );
-  
+
     $p->{return}          ||= 'object';
     $p->{search_type}     ||= 'any';
 
     # after this assignment, @fixed_search is filled with stemmed,
     # lowercased non-STOPWORDS from the terms that the user has given us
 
-    my @fixed_search = map { Lingua::Stem::stem( $_ )->[0] } 
-                       grep { not defined $STOPWORDS{ $_ } } 
-                       map { lc $_ } 
+    my @fixed_search = map { Lingua::Stem::stem( $_ )->[0] }
+                       grep { not defined $STOPWORDS{ $_ } }
+                       map { lc $_ }
                        @{ $p->{search_terms} };
 
     # \%results is indexed by the object identifier coming from the
@@ -109,7 +109,7 @@ sub search_fulltext_index {
                     keys %{ $results };
 
     if ( $p->{return} eq 'raw' )      { return \@frequency }
-    if ( $p->{return} eq 'iterator' ) { return OpenInteract::FullTextIterator->new({ 
+    if ( $p->{return} eq 'iterator' ) { return OpenInteract::FullTextIterator->new({
                                                             %{ $p }, results => \@frequency }) }
     if ( $p->{return} eq 'object' )   { return $class->_fulltext_object_listing( \@frequency, $p ) }
 }
@@ -125,8 +125,8 @@ sub _fulltext_object_listing {
     $p ||= {};
     my ( @object_list );
     foreach my $result ( @{ $result_list } ) {
-        push @object_list, OpenInteract::SPOPS->fetch_object_by_key( 
-                                                     $result->[0], 
+        push @object_list, OpenInteract::SPOPS->fetch_object_by_key(
+                                                     $result->[0],
                                                      { column_group  => COLUMN_GROUP,
                                                        skip_security => $p->{skip_security},
                                                        %{ $p } });
@@ -185,7 +185,7 @@ sub remove_object_from_index {
     my ( $self, $p ) = @_;
     my $object_key = $self->generate_object_key;
     my $R = OpenInteract::Request->instance;
-    $R->DEBUG && $R->scrib( 1, "Trying to remove from index: ", 
+    $R->DEBUG && $R->scrib( 1, "Trying to remove from index: ",
                                ref $self, " (", $self->id, ")" );
     return eval { $self->db_delete({ table => $TABLE_NAME, 
                                      where => 'object_key = ?',
@@ -264,7 +264,7 @@ sub _store_terms {
 # $p->{terms} should be an arrayref of words already stemmed and the
 # STOPWORDS picked out -- it's a clean list
 
-# Return a hashref of information, 
+# Return a hashref of information,
 # { ft_oid => { term => $occur, ... } }
 
 sub _execute_fulltext_search {
@@ -362,21 +362,6 @@ sub _screen_results {
 }
 
 
-# Create a unique identifier for the object from the class and ID (not
-# used anymore, will probably be removed)
-
-#sub _build_fulltext_object_id {
-#    return $_[0]->{tmp_ft_oid} = join '>>', ref $_[0], $_[0]->id;
-#}
-
-
-# Deserialize the unique object identifier (not used anymore, will
-# probably be removed)
-
-#sub _crack_fulltext_object_id {
-#    return split '>>', $_[1];
-#}
-
 1;
 
 __END__
@@ -569,15 +554,6 @@ in the object.
 
 =back
 
-B<_build_fulltext_object_id()>
-
-Returns the string used to uniquely identify an object.
-
-B<_crack_fulltext_object_id( $object_identifier )>
-
-Takes the $object_identifier as created by I<_build_fulltext_object_id> and
-returns a two-element list of the object class and ID.
-
 B<_fulltext_search_terms( \%params )>
 
 Performs the actual search, putting the results into an intermediate
@@ -590,7 +566,7 @@ each term as the value there. For instance, doing a search for
                                          senate => 3 },
            'OpenInteract::BasicPage||/toolkit/congress' => { congress => 8, 
                                                              senate => 1 },
-           ... 
+           ...
   };
 
 =head1 TO DO
