@@ -1,6 +1,6 @@
 package OpenInteract::Handler::BasicPage;
 
-# $Id: BasicPage.pm,v 1.14 2001/08/22 04:21:19 lachoy Exp $
+# $Id: BasicPage.pm,v 1.15 2001/10/12 16:04:14 lachoy Exp $
 
 use strict;
 use SPOPS::Secure  qw( :level );
@@ -9,7 +9,7 @@ use Data::Dumper   qw( Dumper );
 use File::Basename qw();
 
 @OpenInteract::Handler::BasicPage::ISA     = qw( OpenInteract::Handler::GenericDispatcher SPOPS::Secure  );
-$OpenInteract::Handler::BasicPage::VERSION = sprintf("%d.%02d", q$Revision: 1.14 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::BasicPage::VERSION = sprintf("%d.%02d", q$Revision: 1.15 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::BasicPage::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::BasicPage::default_method    = 'listing';
@@ -21,7 +21,8 @@ $OpenInteract::Handler::BasicPage::default_method    = 'listing';
 );
 
 
-use constant MAIN_SCRIPT => '/BasicPage';
+use constant MAIN_SCRIPT     => '/BasicPage';
+use constant NO_PARSE_STATUS => 'no_parse';
 
 # Use this to mark the beginning and end of the "good" content in a
 # page in the filesystem; this allows you to use an HTML editor to
@@ -48,8 +49,8 @@ my $DEFAULT_EXPIRE = 60 * 60 * 24 * 7 * 52;
 sub _get_task {
     my ( $class ) = @_;
     my $R = OpenInteract::Request->instance;
-    return 'show'  if ( $R->{path}->{full}->[0] ne 'BasicPage' );
-    return lc shift @{ $R->{path}->{current} }
+    return 'show'  if ( $R->{path}{full}->[0] ne 'BasicPage' );
+    return lc shift @{ $R->{path}{current} }
                 || $OpenInteract::Handler::BasicPage::default_method;
 }
 
@@ -83,7 +84,7 @@ sub listing {
                                                value        => [ $selected_dir ] });
     }
 
-    $R->{page}->{title} = 'Listing of Documents';
+    $R->{page}{title} = 'Listing of Documents';
     return $R->template->handler( {}, $params,
                                   { db      => 'static_dir_listing',
                                     package => 'static_page' } );
@@ -107,9 +108,9 @@ sub show {
 
     my $location = $p->{location} ||
                    $R->apache->param( 'location' ) ||
-                   $R->{path}->{original};
+                   $R->{path}{original};
 
-    my $storage = $R->CONFIG->{action}->{basicpage}->{storage};
+    my $storage = $R->CONFIG->{action}{basicpage}{storage};
 
     my ( $page_info );
     if ( $page ) {
@@ -150,7 +151,7 @@ sub show {
             # Acrobat Reader) will try to display it, but the *actual*
             # content will be plain old HTML...
 
-            $R->{page}->{content_type} = 'text/html';
+            $R->{page}{content_type} = 'text/html';
 
             if ( $error_type =~ /^security/ ) {
                 return "<h2>Access Forbidden</h2>" .
@@ -171,7 +172,7 @@ sub show {
     # quit processing
 
     if ( $page_info->{send_file} ) {
-        $R->{page}->{send_file} = $page_info->{send_file};
+        $R->{page}{send_file} = $page_info->{send_file};
         $R->DEBUG && $R->scrib( 1, "File being retrieved is not displayable. Set 'send_file' to $page_info->{send_file}" );
         return undef;
     }
@@ -186,10 +187,10 @@ sub show {
 
     if ( $do_edit ) {
         $page ||= $R->static_page->new;
-        $params->{static_page} = $page;  
+        $params->{static_page} = $page;
         $text_params = { db      => 'static_page_form',
                          package => 'static_page' };
-        $R->{page}->{title}   = 'Edit a Document';
+        $R->{page}{title}   = 'Edit a Document';
         my $update_items = eval { $page->fetch_updates( 5 ) } || [];
         foreach my $update_info ( @{ $update_items } ) {
             my $user = eval { $R->user->fetch( $update_info->[0] ) };
@@ -203,19 +204,19 @@ sub show {
 
         unless ( my $is_active = $class->_check_active_date( $page_info ) ) {
             $R->DEBUG && $R->scrib( 1, "Page has been marked as non-active ($is_active); returning error message." );
-            $R->{page}->{title} = 'Page not yet active';
+            $R->{page}{title} = 'Page not yet active';
             return '<h2 align="center">Not Active</h2>' .
                    '<p>Sorry, this page is not yet active.</p>';
         }
 
-        $R->{page}->{title}   = $page_info->{title};
-        $R->{page}->{script} .= $page_info->{script};
+        $R->{page}{title}   = $page_info->{title};
+        $R->{page}{script} .= $page_info->{script};
 
         # Allows the page to define the main template it will use; if
         # the page doesn't define one then the main UI module will use
         # the default
 
-        $R->{page}->{_template_name_} = $page_info->{main_template};
+        $R->{page}{_template_name_} = $page_info->{main_template};
 
         # You can split your page into multiple viewable pages -- see
         # _split_pages() for more info
@@ -226,6 +227,14 @@ sub show {
             $class->_add_object_boxes( $page_info, $p );
         }
     }
+
+    # If the page has told us not to run it through TT, just return
+    # its content.
+
+    if ( $page_info->{_template_status} eq NO_PARSE_STATUS ) {
+        return $page_info->{pagetext};
+    }
+
     return $R->template->handler( {}, $params, $text_params );
 }
 
@@ -235,7 +244,7 @@ sub show {
 sub edit {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-    $R->{page}->{return_url} = '/BasicPage/listing/';
+    $R->{page}{return_url} = '/BasicPage/listing/';
 
     my $apr = $R->apache;
     my $location = $apr->param( 'old_location' );
@@ -344,7 +353,7 @@ sub _retrieve_object {
  
     $location    = $class->_remove_query_string( $location );
     $location    = lc $location;
-    my $home_name = $R->CONFIG->{action}->{basicpage}->{object_home_name};
+    my $home_name = $R->CONFIG->{action}{basicpage}{object_home_name};
  
     my @locations = ( $location );
     if ( $location =~ m|/$| ) {
@@ -397,7 +406,7 @@ sub _retrieve_file {
     my $R = OpenInteract::Request->instance;
     $R->DEBUG && $R->scrib( 1, "Trying to retrieve FILE with location ($location)" );
     $location = $class->_remove_query_string( $location );
-    my $home_name = $R->CONFIG->{action}->{basicpage}->{filesystem_home_name};
+    my $home_name = $R->CONFIG->{action}{basicpage}{filesystem_home_name};
 
     if ( $location =~ m|/$| ) {
         $R->DEBUG && $R->scrib( 1, "Trying to grab a directory; adding $home_name" );
@@ -420,7 +429,7 @@ sub _fetch_file {
 
     my $html_dir = $R->CONFIG->get_dir( 'html' );
     $html_dir =~ s|/$||;
- 
+
     foreach my $location ( @loc ) {
 
         # Security -- remove all '.' from the beginning of the location
@@ -449,8 +458,8 @@ sub _fetch_file {
             $level = eval { $R->basicpage->check_security({
                                  security_object_class => $R->security_object,
                                  object_id             => $location,
-		                         user                  => $R->{auth}->{user},
-                                 group                 => $R->{auth}->{group},
+		                         user                  => $R->{auth}{user},
+                                 group                 => $R->{auth}{group},
                                  hierarchy_separator   => '/' }) };
             $R->scrib( 0, "Error found checking security: $@" )  if ( $@ );
         }
@@ -493,6 +502,9 @@ sub _fetch_file {
         if ( $page_info->{pagetext} =~ s|<boxes>(.*)?</boxes>|| ) {
             $page_info->{boxes}    = $1;
         }
+        if ( $page_info->{pagetext} =~ m|<template_status>(.*)?</template_status>| ) {
+            $page_info->{_template_status}  = $1;
+        }
 
         # Only use the information between the $BODY_DEMARCATION tags
         # (if they exist)
