@@ -1,19 +1,18 @@
 package OpenInteract::Handler::Box;
 
-# $Id: Box.pm,v 1.8 2001/07/25 10:01:18 lachoy Exp $
+# $Id: Box.pm,v 1.12 2001/10/11 03:53:21 lachoy Exp $
 
 use strict;
 use SPOPS::Secure qw( :level );
 use Data::Dumper qw ( Dumper );
 
 @OpenInteract::Handler::Box::ISA     = ();
-$OpenInteract::Handler::Box::VERSION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::Box::VERSION = sprintf("%d.%02d", q$Revision: 1.12 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::Box::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::Box::default_method    = '';
 @OpenInteract::Handler::Box::forbidden_methods = ();
-%OpenInteract::Handler::Box::security = (
-   process_boxes  => SEC_LEVEL_NONE );
+%OpenInteract::Handler::Box::security          = ( process_boxes  => SEC_LEVEL_NONE );
 
 use constant DEFAULT_METHOD       => 'handler';
 use constant DEFAULT_SEPARATOR    => "<br>\n";
@@ -22,10 +21,8 @@ use constant DEFAULT_BOX_WEIGHT   => 5;
 use constant MAX_BOX_WEIGHT       => 100;
 use constant BLANK_SHELL_TEMPLATE => '_blank_';
 
-
 my @BOX_KEYS = qw( class method title weight base_template image image_alt );
 
-
 sub process_boxes {
     my ( $class ) = @_;
 
@@ -37,17 +34,20 @@ sub process_boxes {
     # First, do the system boxes -- this puts box information into the
     # box holding area
 
-    my $system_box_class = $R->CONFIG->{box}->{system_box_handler};
-    $system_box_class->handler();
+    my $system_box_class = $BOX_CONFIG->{system_box_handler};
+    my $system_box_method = $BOX_CONFIG->{system_box_method} || DEFAULT_METHOD;
+    $system_box_class->$system_box_method();
 
     # If a website has boxes that it's adding on every page it can do
     # so in code rather than in a template. Note that this handler can
     # call other handlers as it deems necessary, so that the framework
     # doesn't care about the application-specific usage.
 
-    my $custom_box_class = $R->CONFIG->{box}->{custom_box_handler};
+    my $custom_box_class = $BOX_CONFIG->{custom_box_handler};
     if ( $custom_box_class ) {
-        eval { $custom_box_class->handler() };
+        eval "require $custom_box_class";
+        my $custom_box_method = $BOX_CONFIG->{custom_box_method} || DEFAULT_METHOD;
+        eval { $custom_box_class->$custom_box_method() };
         if ( $@ ) {
             $R->scrib( 0, "Cannot call custom box handler: $custom_box_class. Error: $@" );
         }
@@ -163,13 +163,13 @@ BOX:
         else {
             $shell_params->{content} = $html;
             $shell_params->{label} = $box_info->{title} ||
-                                     $box_info->{params}->{title} ||
+                                     $box_info->{params}{title} ||
                                      $info->{title};
             $shell_params->{label_image_src} = $box_info->{title_image_src} ||
-                                               $box_info->{params}->{title_img_src} ||
+                                               $box_info->{params}{title_img_src} ||
                                                $info->{title_img_src};
             $shell_params->{label_image_alt} = $box_info->{title_image_alt} ||
-                                               $box_info->{params}->{title_img_alt} ||
+                                               $box_info->{params}{title_img_alt} ||
                                                $info->{title_img_alt};
             $R->DEBUG && $R->scrib( 3, "Passing parameters to fill in the box of: ",
                                        Dumper( $shell_params ) );
@@ -208,7 +208,7 @@ OpenInteract::Handler::Box -- Handle input and output for independent "boxes"
 
  # Add a box
 
- my $zip = $R->{auth}->{user}->{zip_code};
+ my $zip = $R->{auth}{user}{zip_code};
  my $box = { name   => 'current_weather',
              weight => 2,
              title  => "Weather in Zip Code $zip",
