@@ -1,13 +1,13 @@
 package OpenInteract::Handler::Group;
 
-# $Id: Group.pm,v 1.4 2001/08/26 14:10:39 lachoy Exp $
+# $Id: Group.pm,v 1.5 2001/09/14 03:22:47 lachoy Exp $
 
 use strict;
 use SPOPS::Secure qw( :level );
 use Data::Dumper  qw( Dumper );
 
 @OpenInteract::Handler::Group::ISA     = qw( OpenInteract::Handler::GenericDispatcher SPOPS::Secure );
-$OpenInteract::Handler::Group::VERSION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::Group::VERSION = sprintf("%d.%02d", q$Revision: 1.5 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::Group::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::Group::default_method    = 'listing';
@@ -29,17 +29,17 @@ sub listing {
     my $R = OpenInteract::Request->instance;
     my $params = { main_script => MAIN_SCRIPT, 
                    error_msg   => $p->{error_msg} };
-    $params->{group_list} = eval { $R->group->fetch_group( { order => 'name' } ); }; 
+    $params->{group_list} = eval { $R->group->fetch_group( { order => 'name' } ); };
     if ( $@ ) {
         $params->{error_msg} .= "\n\nCannot retrieve group listing";
         OpenInteract::Error->set( SPOPS::Error->get );
         $OpenInteract::Error::user_msg = 'Cannot retrieve groups for listing.';
-        $R->throw( { code => 403 } );
+        $R->throw({ code => 403 });
     }
-    $R->{page}->{title} = 'Group Listing';
-    return $R->template->handler( {}, $params, 
-                                  { db      => 'group_list', 
-                                    package => 'base_group' } );
+    $R->{page}{title} = 'Group Listing';
+    return $R->template->handler( {}, $params,
+                                  { package => 'base_group',
+                                    db      => 'group_list' } );
 }
 
 
@@ -52,10 +52,11 @@ sub show {
     if ( $@ ) {
         my $ei = OpenInteract::Error->set( SPOPS::Error->get );
         if ( $ei->{type} eq 'security' ) {
-            $params->{error_msg} .= "You do not have access to view this group. Permission denied -- returning to listing";
+            $params->{error_msg} .= 'You do not have access to view this group. ' .
+                                    'Permission denied -- returning to listing';
         }
         else {
-            $R->throw( { code => 404 } );
+            $R->throw({ code => 404 });
             $params->{error_msg} .= "Error -- most likely a database error -- retrieving group object." .
                                     "Error has been logged. Returning you to listing.";
         }
@@ -66,7 +67,8 @@ sub show {
     # group object, there's no reason to continue
 
     if ( ! $group and $p->{level} < SEC_LEVEL_WRITE ) {
-        $params->{error_msg} = 'Sorry, you do not have access to create a new group object. Returning to listing.';
+        $params->{error_msg} = 'Sorry, you do not have access to create a new ' .
+                               'group object. Returning to listing.';
         return $class->listing( $params );
     }
 
@@ -87,7 +89,7 @@ sub show {
 
     my $mem_users = eval { $group->user } || [];
     $params->{error_msg} .= "Failed to retrieve user listing: $@->{error}"   if ( $@ );
-    my @mem_user_list = map { { id => $_->{user_id}, name => $_->{login_name} } } @{ $mem_users };
+    my @mem_user_list = map { { id => $_->id, name => $_->{login_name} } } @{ $mem_users };
 
     $params->{group} = $group;
     $params->{mem_user_list} = \@mem_user_list;
@@ -105,34 +107,35 @@ sub show {
             $params->{error_msg} = "Cannot retrieve all users. Error logged.";
         }
         $all_user_list ||= [];
-        my %all_nonmem_users = map { $_->{user_id} => $_->{login_name} } 
+        my %all_nonmem_users = map { $_->id => $_->{login_name} }
                                    @{ $all_user_list };
         foreach ( @mem_user_list ) { delete $all_nonmem_users{ $_->{id} } }
         my @all_nonmem_users = map { { id => $_, name => $all_nonmem_users{ $_ } } }
-                               sort { $all_nonmem_users{ $a } cmp $all_nonmem_users{ $b } } 
+                               sort { $all_nonmem_users{ $a } cmp $all_nonmem_users{ $b } }
                                keys %all_nonmem_users;
         $params->{member_field}  = MEMBER_FIELD;
         $params->{all_user_list} = \@all_nonmem_users;
         $tmpl_name = 'group_form';
     }
 
-    $R->{page}->{title}  = 'Group Detail';
+    $R->{page}{title}  = 'Group Detail';
     return $R->template->handler( {}, $params,
-                                  { db      => $tmpl_name,
-                                    package => 'base_group' } );
+                                  { package => 'base_group',
+                                    db      => $tmpl_name } );
 }
 
 
 sub edit {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-    my $params = { error_msg => $p->{error_msg}, main_script => MAIN_SCRIPT };
-    $R->{page}->{return_url} = '/Group/'; 
+    my $params = { error_msg   => $p->{error_msg},
+                   main_script => MAIN_SCRIPT };
+    $R->{page}{return_url} = '/Group/';
 
     # First get the gid from the user and try to fetch
 
     my $apr = $R->apache;
-    my $group_id = $apr->param( 'group_id' ); 
+    my $group_id = $apr->param( 'group_id' );
     my $group = eval { $R->group->fetch( $group_id ); };
     if ( $@ ) {
         OpenInteract::Error->set( SPOPS::Error->get );
@@ -154,9 +157,8 @@ sub edit {
     # all relevant form values for the group (new or old)
 
     $group ||= $R->group->new;
-    foreach my $field ( @{ $R->group->field_list } ) {
-        $group->{ $field } = $apr->param( $field );
-    }
+    $group->{name}  = $apr->param( 'name' );
+    $group->{notes} = $apr->param( 'notes' );
 
     # Try to save; if we can't pass back an error.
 
@@ -164,20 +166,20 @@ sub edit {
     if ( $@ ) {
         OpenInteract::Error->set( SPOPS::Error->get );
         $OpenInteract::Error::user_msg = 'Error saving group object.';
-        $R->throw( { code => 407 } );
+        $R->throw({ code => 407 });
         $params->{error_msg} = "Cannot save group object. Error logged.";
         return $class->show({ group => $group, %{ $params } });
     }
 
     # Now save the members that were specified
-    # 
+    #
     # First get the existing members, then split apart the members
     # specified in the form. Give both pieces of information to
     # the list_process method to separate them out into removals,
     # additions and keepers.
 
-    my $existing_user_list = eval { $group->user } || []; 
-    my @existing_uid = map { $_->{user_id} } @{ $existing_user_list };
+    my $existing_user_list = eval { $group->user } || [];
+    my @existing_uid = map { $_->id } @{ $existing_user_list };
     my @member_uid   = split ';', $apr->param( MEMBER_FIELD );
     $R->DEBUG && $R->scrib( 1, "User IDs retrieved: ", join( '/', @member_uid ) );
     my $mem_info = $group->list_process( \@existing_uid, \@member_uid );
@@ -195,7 +197,8 @@ sub edit {
     # edit the information again; if they didn't have an error,
     # then they'll see that something happened.
 
-    $params->{group} = $group;
+    $params->{status_msg} = 'Group information updated.';
+    $params->{group}      = $group;
     return $class->show( $params );
 }
 
@@ -220,19 +223,20 @@ sub remove {
         }
     }
     unless ( $group ) {
-        return $class->listing( { error_msg => 'Cannot create group for removal: bad ID for object.' } );
+        return $class->listing({ error_msg => 'Cannot create group for removal: bad ID for object.' });
     }
     if ( $group->{tmp_security_level} < SEC_LEVEL_WRITE ) {
-        my $user_msg = 'Sorry, you do not have access to remove this group object. Returning to listing.';
-        return $class->listing( { error_msg => $user_msg } );
+        my $user_msg = 'Sorry, you do not have access to remove this group object. ' .
+                       'Returning to listing.';
+        return $class->listing({ error_msg => $user_msg });
     }
-
     eval { $group->remove };
     if ( $@ ) {
         OpenInteract::Error->set( SPOPS::Error->get );
-        $R->throw( { code => 405 } );
+        $R->throw({ code => 405 });
         $p->{error_msg} = 'Error removing group object! Error logged.';
     }
+    $p->{status_msg} = 'Group removed.';
     return $class->listing( $p );
 }
 
