@@ -1,20 +1,20 @@
 package OpenInteract::Handler::Package;
 
-# $Id: Package.pm,v 1.5 2001/07/13 14:37:04 lachoy Exp $
+# $Id: Package.pm,v 1.6 2001/09/22 16:22:37 lachoy Exp $
 
 use strict;
-use SPOPS::Secure qw( :level );
 use Data::Dumper  qw( Dumper );
+use OpenInteract::Handler::GenericDispatcher qw( DEFAULT_SECURITY_KEY );
+use SPOPS::Secure qw( :level );
 
 @OpenInteract::Handler::Package::ISA               = qw( OpenInteract::Handler::GenericDispatcher SPOPS::Secure );
-$OpenInteract::Handler::Package::VERSION           = sprintf("%d.%02d", q$Revision: 1.5 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::Package::VERSION           = sprintf("%d.%02d", q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
 $OpenInteract::Handler::Package::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::Package::default_method    = 'listing';
 @OpenInteract::Handler::Package::forbidden_methods = ();
-%OpenInteract::Handler::Package::security          = ( 
- listing => SEC_LEVEL_READ,  show   => SEC_LEVEL_WRITE,    
- edit    => SEC_LEVEL_WRITE, remove => SEC_LEVEL_WRITE,
-);
+%OpenInteract::Handler::Package::security          = (
+ DEFAULT_SECURITY_KEY() => SEC_LEVEL_WRITE,
+ listing                => SEC_LEVEL_READ );
 
 use constant MAIN_SCRIPT => '/Package';
 
@@ -24,28 +24,27 @@ use constant MAIN_SCRIPT => '/Package';
 sub listing {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-    my $params = { main_script => MAIN_SCRIPT, 
+    my $params = { main_script => MAIN_SCRIPT,
                    error_msg   => $p->{error_msg} };
 
     # First retrieve the packages from this app
 
-    my $website_dir = $R->CONFIG->{dir}->{base};
+    my $website_dir = $R->CONFIG->{dir}{base};
     $R->DEBUG && $R->scrib( 1, "Trying to open repository in website dir ($website_dir)" );
-    my $website_repository  = $R->repository->fetch( 
+    my $website_repository  = $R->repository->fetch(
                                        undef, { directory => $website_dir } );
     $params->{website_pkg}  = $website_repository->fetch_all_packages();
 
     # Next, retrieve the packages from base installation
 
-    my $base_install_dir = $R->CONFIG->{dir}->{interact};
+    my $base_install_dir = $R->CONFIG->{dir}{interact};
     $R->DEBUG && $R->scrib( 1, "Trying to open repository in base installation dir ($base_install_dir)" );
-    my $base_repository     = $R->repository->fetch( 
+    my $base_repository     = $R->repository->fetch(
                                        undef, { directory => $base_install_dir } );
     $params->{interact_pkg} = $base_repository->fetch_all_packages();
-    $R->{page}->{title} = "Package Listing";
-    return $R->template->handler( {}, $params, 
-                                  { db      => 'package_listing', 
-                                    package => 'base' } );
+    $R->{page}{title} = "Package Listing";
+    return $R->template->handler( {}, $params,
+                                  { name => 'base::package_listing' } );
 }
 
 
@@ -55,20 +54,20 @@ sub listing {
 sub show {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-    my $params = { main_script => MAIN_SCRIPT, 
+    my $params = { main_script => MAIN_SCRIPT,
                    error_msg   => $p->{error_msg} };
     my $do_edit = ( $R->apache->param( 'edit' ) and $p->{level} >= SEC_LEVEL_WRITE );
 
     $params->{scope} = $R->apache->param( 'scope' ) || 'local';
-    my $base_dir = ( $params->{scope} eq 'interact' ) 
-                     ? $R->CONFIG->{dir}->{interact} 
-                     : $R->CONFIG->{dir}->{base};
+    my $base_dir = ( $params->{scope} eq 'interact' )
+                     ? $R->CONFIG->{dir}{interact}
+                     : $R->CONFIG->{dir}{base};
     my $repository = $R->repository->fetch( undef, { directory => $base_dir } );
 
     my $name    = $R->apache->param( 'name' );
     my $version = $R->apache->param( 'version' );
-    my $pkg = $p->{pkg} || 
-              $repository->fetch_package_by_name({ name    => $name, 
+    my $pkg = $p->{pkg} ||
+              $repository->fetch_package_by_name({ name    => $name,
                                                    version => $version });
     unless ( $pkg ) {
         my $error_msg = "Sorry, couldn't find a package matching the criteria you specified.";
@@ -78,9 +77,8 @@ sub show {
         $params->{error_msg} .= "Found a working package, but it wasn't the version you requested.";
     }
     $params->{pkg} = $pkg;
-    return $R->template->handler( {}, $params, 
-                                  { db      => 'package_detail', 
-                                    package => 'base' } );
+    return $R->template->handler( {}, $params,
+                                  { name => 'base::package_detail' } );
 }
 
 sub edit {
@@ -90,11 +88,11 @@ sub edit {
     my $name    = $R->apache->param( 'name' );
     my $version = $R->apache->param( 'version' );
     my $scope   = $R->apache->param( 'scope' );
-    my $base_dir = ( $scope eq 'interact' ) ? $R->CONFIG->{dir}->{interact} : $R->CONFIG->{dir}->{base};
+    my $base_dir = ( $scope eq 'interact' ) ? $R->CONFIG->{dir}{interact} : $R->CONFIG->{dir}{base};
     my $repository = $R->repository->fetch( undef, { directory => $base_dir,
                                                      perm      => 'write' } );
-  
-    my $pkg = $repository->fetch_package_by_name({ name    => $name, 
+
+    my $pkg = $repository->fetch_package_by_name({ name    => $name,
                                                    version => $version });
     if ( $@ or ! $pkg ) {
         my $error = $@ || "No package found using $name-$version";
@@ -106,7 +104,7 @@ sub edit {
         $pkg->{ $field } = $R->apache->param( $field );
     }
     $pkg->{last_updated_on} = $repository->now;
-    $pkg->{last_updated_by} = join( ' ', $R->{auth}->{user}->{login_name}, "<$R->{auth}->{user}->{email}>" );
+    $pkg->{last_updated_by} = join( ' ', $R->{auth}{user}{login_name}, "<$R->{auth}{user}{email}>" );
     $repository->save_package( $pkg );
     eval { $repository->save() };
     if ( $@ ) {
