@@ -1,6 +1,6 @@
 package OpenInteract::Handler::Security;
 
-# $Id: Security.pm,v 1.8 2001/08/26 03:30:26 lachoy Exp $
+# $Id: Security.pm,v 1.11 2001/09/28 18:59:52 lachoy Exp $
 
 use strict;
 use Data::Dumper             qw( Dumper );
@@ -8,8 +8,8 @@ use SPOPS::Secure            qw( :level :scope );
 use SPOPS::Secure::Hierarchy qw( $ROOT_OBJECT_NAME );
 use OpenInteract::Handler::GenericDispatcher qw( DEFAULT_SECURITY_KEY );
 
-@OpenInteract::Handler::Security::ISA     = qw( OpenInteract::Handler::GenericDispatcher );
-$OpenInteract::Handler::Security::VERSION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
+@OpenInteract::Handler::Security::ISA     = qw( OpenInteract::Handler::GenericDispatcher SPOPS::Secure );
+$OpenInteract::Handler::Security::VERSION = sprintf("%d.%02d", q$Revision: 1.11 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::Security::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::Security::default_method    = 'listing';
@@ -356,7 +356,7 @@ sub _fetch_description {
 
 # Message for when a class isn't derived from SPOPS::Secure
 
-sub _cannot_return_msg {
+sub _cannot_secure_msg {
     my ( $object_class ) = @_;
     return <<CANNOTSECURE;
 <h2>Object Not Securable</h2>
