@@ -1,6 +1,6 @@
 package OpenInteract::Handler::SiteSearch;
 
-# $Id: SiteSearch.pm,v 1.10 2001/07/15 20:54:42 lachoy Exp $
+# $Id: SiteSearch.pm,v 1.11 2001/10/01 17:13:51 lachoy Exp $
 
 use strict;
 use Data::Dumper  qw( Dumper );
@@ -12,7 +12,7 @@ BEGIN {
     };
 }
 @OpenInteract::Handler::SiteSearch::ISA               = qw( OpenInteract::Handler::GenericDispatcher );
-$OpenInteract::Handler::SiteSearch::VERSION           = sprintf("%d.%02d", q$Revision: 1.10 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::SiteSearch::VERSION           = sprintf("%d.%02d", q$Revision: 1.11 $ =~ /(\d+)\.(\d+)/);
 $OpenInteract::Handler::SiteSearch::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::SiteSearch::default_method    = 'search';
 @OpenInteract::Handler::SiteSearch::forbidden_methods = ();
@@ -24,7 +24,7 @@ use constant MAIN_SCRIPT      => '/SiteSearch';
 sub search {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-    
+
     my $apr = $R->apache;
     my $params = { main_script => MAIN_SCRIPT,
                    error_msg   => $p->{error_msg} };
@@ -33,14 +33,12 @@ sub search {
     $params->{keywords}  = $apr->param( 'keywords' );
 
     unless ( $params->{keywords} or $params->{search_id} ) {
-        return $R->template->handler( {}, {}, 
-                                      { db      => 'no_keywords',
-                                        package => 'full_text' } );
+        return $R->template->handler( {}, {}, { name => 'full_text::no_keywords' } );
     }
 
     $params->{current_page} = $apr->param( 'page' ) || 1;
     my $hits_per_page = DEFAULT_PER_PAGE;
-    my ( $min, $max ) = OpenInteract::ResultsManage->find_page_boundaries( 
+    my ( $min, $max ) = OpenInteract::ResultsManage->find_page_boundaries(
                                                   $params->{current_page}, $hits_per_page );
 
     # Run the search for the first time and save the results
@@ -50,11 +48,11 @@ sub search {
         $params->{search_type} = $apr->param( 'search_type' ) || 'all';
         my @include_classes = $apr->param( 'include_class' );
         $R->DEBUG && $R->scrib( 1, "Searching for ($params->{keywords}) with ($params->{search_type})" );
-        my $search_results = OpenInteract::FullText->search_fulltext_index({ 
-                                      search_terms    => \@terms,
-                                      search_type     => $params->{search_type},
-                                      return          => 'raw' }) || [];
-        $R->DEBUG && $R->scrib( 2, "List of found object keys and frequencies:", 
+        my $search_results = OpenInteract::FullText->search_fulltext_index({
+                                      search_terms => \@terms,
+                                      search_type  => $params->{search_type},
+                                      return       => 'raw' }) || [];
+        $R->DEBUG && $R->scrib( 2, "List of found object keys and frequencies:",
                                    Dumper( $search_results ) );
 
         # Persist the raw results and get the ID so we can use them
@@ -70,15 +68,14 @@ sub search {
     my $results = OpenInteract::ResultsManage->new({ search_id => $params->{search_id},
                                                      min => $min, max => $max });
     $results->retrieve;
-    $params->{search_iterator} = OpenInteract::FullTextIterator->new({ 
+    $params->{search_iterator} = OpenInteract::FullTextIterator->new({
                                                   saved_results => $results->{result_list} });
 
     $params->{total_hits}  = $results->{num_records};
     $params->{total_pages} = $results->find_total_page_count( $params->{total_hits} );
     $R->{page}->{title} = 'Search Results';
     return $R->template->handler( {}, $params,
-                                  { db      => 'search_results',
-                                    package => 'full_text' } );
+                                  { name => 'full_text::search_results' } );
 }
 
 1;
