@@ -1,12 +1,12 @@
 package OpenInteract::Handler::SiteTemplate;
 
-# $Id: SiteTemplate.pm,v 1.6 2001/07/13 15:08:29 lachoy Exp $
+# $Id: SiteTemplate.pm,v 1.7 2001/07/31 15:28:37 lachoy Exp $
 
 use strict;
 use SPOPS::Secure qw( :level );
 
 @OpenInteract::Handler::SiteTemplate::ISA     = qw( OpenInteract::Handler::GenericDispatcher  SPOPS::Secure );
-$OpenInteract::Handler::SiteTemplate::VERSION = sprintf("%d.%02d", q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::SiteTemplate::VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::SiteTemplate::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::SiteTemplate::default_method    = 'listing';
@@ -47,8 +47,18 @@ sub show {
                    package_list => $R->CONFIG->{package_list} };
 
     my $template = $p->{template};
-    my $template_id = $R->apache->param( 'template_id' );
-    $template  ||= eval { $R->site_template->fetch( $template_id ) };
+    unless ( $template ) {
+        my $template_id = $R->apache->param( 'template_id' );
+        if ( $template_id ) {
+            $template = eval { $R->site_template->fetch( $template_id ) };
+        }
+        else {
+            my $full_name = $R->apache->param( 'template_name' );
+            if ( $full_name ) {
+                $template = eval { $R->site_template->fetch_by_name( $full_name ) };
+            }
+        }
+    }
     if ( $@ ) { 
         OpenInteract::Error->set( SPOPS::Error->get );
         $R->throw( { code => 404 } );   
