@@ -1,21 +1,23 @@
 package OpenInteract::Handler::User;
 
-# $Id: User.pm,v 1.4 2001/07/13 15:16:14 lachoy Exp $
+# $Id: User.pm,v 1.6 2001/09/14 12:22:34 lachoy Exp $
 
 use strict;
 use Data::Dumper  qw( Dumper );
+use OpenInteract::Handler::GenericDispatcher qw( DEFAULT_SECURITY_KEY );
 use SPOPS::Secure qw( :level :scope );
 
 @OpenInteract::Handler::User::ISA     = qw( OpenInteract::Handler::GenericDispatcher  SPOPS::Secure );
-$OpenInteract::Handler::User::VERSION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::User::VERSION = sprintf("%d.%02d", q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::User::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::User::default_method    = 'search_form';
-@OpenInteract::Handler::User::forbidden_methods = qw( 
+@OpenInteract::Handler::User::forbidden_methods = qw(
       _search_params _search_params_ldap _search_params_dbi );
 %OpenInteract::Handler::User::security          = (
-      listing => SEC_LEVEL_READ, edit   => SEC_LEVEL_WRITE,
-      show    => SEC_LEVEL_READ, remove => SEC_LEVEL_WRITE );
+      DEFAULT_SECURITY_KEY() => SEC_LEVEL_READ,
+      edit   => SEC_LEVEL_WRITE,
+      remove => SEC_LEVEL_WRITE );
 
 use constant MAIN_SCRIPT => '/User';
 
@@ -39,7 +41,7 @@ sub listing {
         OpenInteract::Error->set( SPOPS::Error->get );
         $R->throw({ code => 404 });
     }
-    $R->{page}->{title} = 'User Listing';
+    $R->{page}{title} = 'User Listing';
     return $R->template->handler( {}, $params,
                                   { name => 'base_user::user_list' } );
 }
@@ -82,9 +84,7 @@ sub _search_params_ldap {
         my $ldap_prop = $field_map->{ $field };
         push @filter_chunk, "($ldap_prop=*$search->{ $field }*)";
     }
-    unless ( scalar @filter_chunk ) {
-        return {};
-    }
+    return {} unless ( scalar @filter_chunk );
     return { filter => '(&' . join( '', @filter_chunk ) . ')' };
 }
 
@@ -96,17 +96,18 @@ sub show {
     my $R = OpenInteract::Request->instance;
 
     # Add this to the hashref below if you want users to be able to choose themes
-    # theme_list => eval { $R->theme->fetch_group( { order => 'title' } ) } 
+    # theme_list => eval { $R->theme->fetch_group( { order => 'title' } ) },
 
-    my $params = { main_script => MAIN_SCRIPT, 
+    my $params = { main_script => MAIN_SCRIPT,
                    error_msg   => $p->{error_msg} };
     $params->{user} = $p->{user};
     unless ( $params->{user} ) {
-        my $user_id = $R->apache->param( 'user_id' ) || $R->apache->param( 'uid' ); 
+        my $user_id = $R->apache->param( 'user_id' ) || $R->apache->param( 'uid' );
         $params->{user}  = eval { $R->user->fetch( $user_id ) };
         if ( $@ ) {
             OpenInteract::Error->set( SPOPS::Error->get );
-            $R->throw( { code => 404 } );
+            $R->throw({ code => 404 });
+            $R->scrib( 0, "Cannot retrieve user ($user_id) ($@) $SPOPS::Error::system_msg" );
         }
     }
 
@@ -115,15 +116,16 @@ sub show {
 
     if ( ! $params->{user} and $p->{level} < SEC_LEVEL_WRITE ) {
         my $msg = 'Sorry, you do not have access to create a new user object. Returning to listing.';
-        return $class->listing( { error_msg => $msg } );
+        $R->scrib( 0, "No permission to create new user." );
+        return $class->listing({ error_msg => $msg });
     }
-  
-    my $obj_level = ( $params->{user} ) 
-                      ? $params->{user}->{tmp_security_level} 
+
+    my $obj_level = ( $params->{user} )
+                      ? $params->{user}{tmp_security_level}
                       : SEC_LEVEL_WRITE;
 
-    $params->{user} ||= $R->user->new( { theme_id => $R->{theme}->{theme_id} } );
-    $R->DEBUG && $R->scrib( 1, "Levels  $params->{user}->{user_id}: $obj_level/$p->{level}" );
+    $params->{user} ||= $R->user->new({ theme_id => $R->{theme}{theme_id} });
+    $R->DEBUG && $R->scrib( 1, "Levels  $params->{user}{user_id}: $obj_level/$p->{level}" );
     $params->{group_list} = eval { $params->{user}->group };
 
     my $tmpl_name = 'user_detail';
@@ -131,23 +133,23 @@ sub show {
     if ( $p->{level} == SEC_LEVEL_WRITE and $obj_level == SEC_LEVEL_WRITE ) {
         $tmpl_name = 'user_form';
     }
-  
-    $R->{page}->{title} = 'View User Detail';
-    return $R->template->handler( {}, $params, 
-                                  { db      => $tmpl_name, 
-                                    package => 'base_user' } );
+
+    $R->{page}{title} = 'View User Detail';
+    return $R->template->handler( {}, $params,
+                                  { package => 'base_user',
+                                    db      => $tmpl_name  } );
 }
 
 
 sub edit {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-    $R->{page}->{return_url} = '/User/';
+    $R->{page}{return_url} = '/User/';
     my $apr = $R->apache;
 
     my $user_id = $apr->param( 'user_id' );
     $R->DEBUG && $R->scrib( 1, "Trying to modify ID <<$user_id>>" );
-  
+
     my $is_new = 0;
     my $user = $p->{user} ||
                eval { $R->user->fetch( $user_id ) };
@@ -155,7 +157,7 @@ sub edit {
         OpenInteract::Error->set( SPOPS::Error->get );
         $R->throw( { code => 404 } );
     }
-  
+
     my $obj_level = ( $user ) ? $user->{tmp_security_level} : SEC_LEVEL_WRITE;
     if ( $obj_level < SEC_LEVEL_WRITE ) {
         my $user_msg = 'Sorry, you do not have access to modify this user object. Returning to listing.';
@@ -173,26 +175,27 @@ sub edit {
     # Uncomment the code if you are allowing users to change their own themes
     # Track this so we can change the theme if the user changed it.
 
-    # my $pre_theme_id = $user->{theme_id} || $R->{theme}->{theme_id};
- 
-    foreach my $field ( @{ $R->user->field_list } ) {
-        next if ( $field eq 'password' or $field eq 'user_id' );
+    # my $pre_theme_id = $user->{theme_id} || $R->{theme}{theme_id};
+
+    my @field_list = qw( first_name last_name email title login_name notes );
+    foreach my $field ( @field_list ) {
         $user->{ $field } = $apr->param( $field );
     }
     my $password = $apr->param( 'password' );
     if ( $password ) {
         $R->DEBUG && $R->scrib( 1, "User asked to change password. Changing." );
-        $password = $R->user->crypt_it( $password ) if ( $R->CONFIG->{login}->{crypt_password} );;
+        $password = $R->user->crypt_it( $password ) if ( $R->CONFIG->{login}{crypt_password} );
         $user->{password} = $password;
     }
-  
+
     eval { $user->save };
     if ( $@ ) {
         OpenInteract::Error->set( SPOPS::Error->get );
         $OpenInteract::Error::user_msg = 'Error saving user object.';
-        $R->throw( { code => 407 } );
-        return $class->show( { user => $user, 
-                               error_msg => 'Cannot save user object! See error log for details...' } );
+        $R->throw({ code => 407 });
+        $R->scrib( 0, "Cannot save user object: $SPOPS::Error::system_msg" );
+        return $class->show({ user      => $user,
+                              error_msg => 'Cannot save user object! See error log for details...' });
     }
 
     # If this is a new user, allow the user to edit his/her own record;
@@ -201,26 +204,27 @@ sub edit {
     # also make the user a member of group 'public'
 
     if ( $is_new ) {
-        my $site_admin_id = $R->CONFIG->{default_objects}->{site_admin_group};
-        eval { 
+        my $site_admin_id = $R->CONFIG->{default_objects}{site_admin_group};
+        eval {
             $user->set_security({ scope => SEC_SCOPE_USER,
                                   level => SEC_LEVEL_WRITE,
-                                  scope_id => $user->{user_id} });
+                                  scope_id => $user->id });
             $user->make_public();
         };
     }
-  
+
 
     # Uncomment the code if you are allowing users to change their own themes
     # If the user changed his/her theme, show it immediately
 
-# if ( $user->{user_id} == $R->{auth}->{user}->{user_id} and 
+# if ( $user->{user_id} == $R->{auth}{user}{user_id} and
 #      $user->{theme_id} != $pre_theme_id ) {
 #   $R->{theme} = eval { $R->theme->fetch( $user->{theme_id} ) };
 # }
 
-    $p->{user} = $user;
-    return $class->show( $p ); 
+    $p->{user}       = $user;
+    $p->{status_msg} = 'User information updated.';
+    return $class->show( $p );
 }
 
 
@@ -229,18 +233,19 @@ sub remove {
     my $R = OpenInteract::Request->instance;
     if ( my $user_id = $R->apache->param( 'user_id' ) ) {
         my $user = eval { $R->user->fetch( $user_id ) };
-        if ( $@ or ! $user ) {     
-            return $class->listing( { error_msg => 'Cannot remove user -- object not created properly.' } );
+        if ( $@ or ! $user ) {
+            return $class->listing({ error_msg => 'Cannot remove user -- object not created properly.' });
         }
         if ( $user->{tmp_security_level} < SEC_LEVEL_WRITE ) {
             my $user_msg = 'Sorry, you do not have access to remove this user object. Returning to listing.';
-            return $class->listing( { error_msg => $user_msg } );   
+            return $class->listing({ error_msg => $user_msg });
         }
-    
+
         eval { $user->remove };
         if ( $@ ) {
             OpenInteract::Error->set( SPOPS::Error->get );
-            $R->throw( { code => 405 } );
+            $R->throw({ code => 405 });
+            $R->scrib( 0, "Failed to remove user object: ($@) $SPOPS::Error::system_msg" );
             $p->{error_msg} = 'Cannot remove user object! See error log for details...';
         }
     }
