@@ -1,17 +1,14 @@
 package OpenInteract::ResultsManage;
 
-# $Id: ResultsManage.pm,v 1.5 2001/07/16 14:23:08 lachoy Exp $
+# $Id: ResultsManage.pm,v 1.9 2001/10/11 03:17:27 lachoy Exp $
 
 use strict;
 use Data::Dumper  qw( Dumper );
+use IO::File;
 use SPOPS::Utility;
 
-BEGIN {
-    eval { require OpenInteract::ResultsIterator };
-}
-
 @OpenInteract::ResultsManage::ISA     = ();
-$OpenInteract::ResultsManage::VERSION = sprintf("%d.%02d", q$Revision: 1.5 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::ResultsManage::VERSION = sprintf("%d.%02d", q$Revision: 1.9 $ =~ /(\d+)\.(\d+)/);
 
 use constant FILENAME_WIDTH => 20;
 
@@ -27,11 +24,9 @@ my @VALID_INIT_KEYS  = qw( search_id min max );
 sub new {
     my ( $pkg, $p ) = @_;
     my $class = ref $pkg || $pkg;
-    my $data = $p || {};
-    foreach my $key ( @VALID_INIT_KEYS ) {
-        $data->{ $key } = $p->{ $key };
-    }
-    return bless( $data, $class );
+    my %data = map { $_ => $p->{ $_ } } @VALID_INIT_KEYS;
+    $data{results_dir} ||= $class->get_results_dir;
+    return bless( \%data, $class );
 }
 
 
@@ -39,316 +34,149 @@ sub new {
 
 sub clear {
     my ( $self ) = @_;
-    while ( my ( $k, $v ) = each %{ $self } ) {
-        undef $self->{ $k };
-    }
+    $self->{ $_ } = undef  for ( keys %{ $self } );
     return $self;
 }
 
 
-# Save given results to a file
+########################################
+# SAVE RESULTS
+########################################
+
+# Save given results to a results file and a meta file
 
 sub save {
-    my ( $self, $identifier_list, $p ) = @_;
-    $p ||= {};
+    my ( $self, $to_save, $p ) = @_;
     my $R = OpenInteract::Request->instance;
     $R->DEBUG && $R->scrib( 1, "Trying to save search results." );
 
-    # Generate a search ID and get the filename, then lock it and open it
+    my %params = ( force_mixed => $p->{force_mixed},
+                   extra       => $p->{extra},
+                   extra_name  => $p->{extra_name} );
 
-    my ( $search_id, $filename ) = $self->get_persistent_info();
-    $self->results_lock( $filename );
-    eval { open( RESULTS, "> $filename" ) || die "Cannot open for writing ($filename): $!\n" };
-    if ( $@ ) { 
+    # First ensure we have a results directory in the object
+
+    $self->{results_dir} ||= $self->get_results_dir;
+
+    # Generate a search ID and get the filename, then lock the
+    # filename from further use while we're working
+
+    $params{search_id}= $self->generate_search_id;
+    $self->results_lock( $params{search_id} );
+    my ( $num_records );
+    my $out      = IO::File->new();
+    my $meta_out = IO::File->new();
+
+    # First write out the actual data, then write out the metadata
+
+    eval {
+        my $results_file = $self->build_results_filename( $params{search_id} );
+        $out->open( "> $results_file" )
+                    || die "Cannot open for writing ($results_file): $!\n";
+        my $results_info = $self->persist( $out, $to_save, \%params );
+        $out->close();
+
+        my $meta_file = $self->build_meta_filename( $params{search_id} );
+        $meta_out->open( "> $meta_file" )
+                         || die "Cannot open for writing ($meta_file): $!\n";
+        $self->persist_meta( $meta_out, $results_info, \%params );
+        $meta_out->close();
+        $num_records = $results_info->{num_records};
+    };
+
+    # If we find an error anywhere along the way, be sure the files
+    # are closed (paranoid, since falling out of scope should do it),
+    # clear the lockfile and die.
+
+    if ( $@ ) {
         $R->scrib( 0, "Search result save failure. $@" );
-        close( RESULTS );
-        $self->results_clear( $filename );
+        $out->close();
+        $meta_out->close();
+        $self->results_clear( $params{search_id} );
         die "Search result save failure: $@\n";
     }
 
+    # Clear out the lockfile
+
+    $self->results_unlock( $params{search_id} );
+
+    # Set various information into the object
+
+    $R->DEBUG && $R->scrib( 1, "($num_records) results saved ok." );
+    $self->{num_records}      = $num_records;
+    return $self->{search_id} = $params{search_id};
+}
+
+
+sub persist {
+    my ( $self, $out, $to_save, $params ) = @_;
+    die "Item to be saved must be a reference!\n" unless ( ref $to_save );
+    if ( ref $to_save eq 'ARRAY' ) {
+        return $self->persist_list( $out, $to_save, $params );
+    }
+    elsif ( UNIVERSAL::isa( $to_save, 'SPOPS::Iterator' ) ) {
+        return $self->persist_iterator( $out, $to_save, $params );
+    }
+    die "Item to be saved must either be arrayref or Iterator!\n";
+}
+
+
+sub persist_list {
+    my ( $self, $out, $to_save, $params ) = @_;
+
     # See if the resultset is homogenous -- if the user wants it mixed
     # then keep it that way, otherwise look at the data passed in to
     # save
 
-    $p->{class} = $MIXED_IDENTIFIER if ( $p->{force_mixed} );
-    $p->{class} ||= $self->review_results_class( $identifier_list );
+    my %info = ( record_class => undef );
+    $info{record_class}   = $MIXED_IDENTIFIER if ( $params->{force_mixed} );
+    $info{record_class} ||= $self->review_results_class( $to_save );
 
-    my $has_extra = 0;
-    my $extra_name = undef;
-    if ( ref $p->{extra} eq 'ARRAY' ) {
-        $has_extra = ( ref $p->{extra}->[0] eq 'ARRAY' )
-                       ? scalar @{ $p->{extra}->[0] } : 1;
-        $p->{extra_name} ||= [];
-        $extra_name = join( $EXTRA_NAME_SEP, @{ $p->{extra_name} } );
+    # 'extra' items are additional pieces of information included with
+    # each record; they can only be saved with non-iterators
+
+    if ( ref $params->{extra} eq 'ARRAY' ) {
+        $info{has_extra}  = ( ref $params->{extra}->[0] eq 'ARRAY' )
+                              ? scalar @{ $params->{extra}->[0] } : 1;
+        $info{extra_name} = $params->{extra_name} || [];
     }
-    my $num_records = scalar @{ $identifier_list };
-    print RESULTS join( $RECORD_SEP, scalar localtime, $num_records, 
-                                     $p->{class}, $has_extra, $extra_name ), "\n";
-    my $count = 0;
+
 
     # Go through the items to save and derive a value and class for
     # each -- it's ok if the class is blank since that means we're
     # just saving raw data
 
-    foreach my $item ( @{ $identifier_list } ) {
-        my ( $item_class, $item_value ) = ( ref $item ) 
-                                            ? ( ref $item, $item->id )
-                                            : ( $p->{class}, $item );
-        $item_class = '' if ( $item_class eq $MIXED_IDENTIFIER );
-        my @result_info = ( $item_class, $item_value );
-        if ( $has_extra ) {
-            if ( ref $p->{extra}->[ $count ] eq 'ARRAY' ) {
-                push @result_info, @{ $p->{extra}->[ $count ] };
+    my $count = 0;
+    foreach my $item ( @{ $to_save } ) {
+        my @result_info = ( ref $item )
+                            ? ( ref $item, $item->id )
+                            : ( $info{record_class}, $item );
+        $result_info[0] = '' if ( $info{record_class} eq $MIXED_IDENTIFIER );
+        if ( $info{has_extra} ) {
+            if ( ref $params->{extra}->[ $count ] eq 'ARRAY' ) {
+                push @result_info, @{ $params->{extra}->[ $count ] };
             }
             else {
-                push @result_info, $p->{extra}->[ $count ];
+                push @result_info, $params->{extra}->[ $count ];
             }
         }
-        print RESULTS join( $RECORD_SEP, @result_info ), "\n";
+        $out->print( join( $RECORD_SEP, @result_info ), "\n" );
         $count++;
     }
-    close( RESULTS );
-    $self->results_unlock( $filename );
-    $R->DEBUG && $R->scrib( 1, scalar @{ $identifier_list }, "search results saved ok." );
-    $self->{num_records} = $num_records;
-    return $self->{search_id} = $search_id;
-}
-
-
-# Returns either ( \@classes, \@ids, $num_records ) or \@ids depending
-# on context
-
-sub retrieve {
-    my ( $self, $p ) = @_;
-    unless ( $self->{search_id} ) {
-        die "Cannot retrieve results without a search_id! Please set at object ",
-            "initialization or as a property of the object before running retrieve().\n";
-    }
-
-    # 'min' and 'max' can be properties or passed in
-
-    $self->{min} ||= $p->{min};
-    $self->{max} ||= $p->{max};
-
-    # Clear out the number of records
-
-    $self->{num_records} = 0;
-
-    my $R = OpenInteract::Request->instance;
-    $R->DEBUG && $R->scrib( 1, "Retrieving raw search results for ID ($self->{search_id})" );
-    $self->assign_results_to_object( $self->retrieve_raw_results( $p ) );
-
-    # If they asked for an iterator return it, but first clear out any
-    # min/max values since they've already been preselected
-
-    if ( $p->{return} eq 'iterator' ) {
-        $self->{min} = $self->{max} = $p->{min} = $p->{max} = 0;
-        return $self->retrieve_iterator( $p );
-    }
-    return $self;
-}
-
-
-# Note that this only works on saved SPOPS objects
-
-sub retrieve_iterator {
-    my ( $self, $p ) = @_;
-
-    # 'min' and 'max' can be properties or passed in
-
-    $self->{min}         ||= $p->{min};
-    $self->{max}         ||= $p->{max};
-
-    my $R = OpenInteract::Request->instance;
-    $R->DEBUG && $R->scrib( 1, "Retrieving search iterator for ID ($self->{search_id})" );
-    unless ( $self->{result_list} ) {
-        $self->assign_results_to_object( $self->retrieve_raw_results( $p ) );
-    }
-    unless ( $self->{record_class} ) {
-        die "Cannot create iterator! Search results were not saved with a classname.",
-            "(Search ID: $self->{search_id})\n";
-    }
-    if ( $self->{record_class} eq $MIXED_IDENTIFIER ) {
-        $self->{min} = $self->{max} = 0;
-        return OpenInteract::ResultsIterator->new({ results       => $self,
-                                                    skip_security => $p->{skip_security} });
-    }
-    else {
-        return $self->{record_class}->fetch_iterator({ id_list => $self->get_id_list() });
-    }
-}
-
-
-# Take the result list internally and return a list of all IDs
-
-sub get_id_list {
-    my ( $self ) = @_;
-    unless ( $self->{result_list} ) {
-        die "Results not yet saved for this object!\n"
-    }
-    return [ map { $_->{id} } @{ $self->{result_list} } ];
-}
-
-
-########################################
-# PAGING METHODS
-########################################
-
-
-# Note that $item can be a class or an object
-
-sub find_total_page_count {
-    my ( $item, $per_page, $p_num_records ) = @_;
-    my $num_records = ( ref $item and $item->{num_records} ) 
-                        ? $item->{num_records} : $p_num_records;
-    return 0 unless ( $per_page and $num_records );
-    my $num_pages = $num_records / $per_page;
-    return ( int $num_pages != $num_pages ) ? int( $num_pages ) + 1 : int $num_pages;
-}
-
-
-sub find_page_boundaries {
-    my ( $class, $page_num, $per_page ) = @_;
-    return ( 0, 0 ) unless ( $page_num and $per_page );
-    my $max = $page_num * $per_page;
-    my $min = $max - $per_page;
-    return ( $min, $max );
-}
-
-
-
-##################################################
-# INTERNAL METHODS
-##################################################
-
-
-# Returns a hashref of information about the results, including the ID
-# list, the class, results save time. Pass in min/max in the hashref
-# (second arg) to have the results be paged.
-
-sub retrieve_raw_results {
-    my ( $self, $p ) = @_;
-    my $R = OpenInteract::Request->instance;
-    die "No search_id defined in object!\n" unless ( $self->{search_id} );
-    my $filename = $self->build_results_filename();
-    eval { open( RESULTS, $filename ) || die "Cannot open ($filename): $!\n" };
-    if ( $@ ) {
-        $R->scrib( 0, "Search result retrieval failure. $@" );
-        return undef;
-    }
-    my %info = ();
-    my @metadata_order = qw( date num_records record_class num_extra tmp_extra_name );
-    my $first_line = <RESULTS>;
-    chomp $first_line;
-    @info{ @metadata_order } = split( $RECORD_SEP, $first_line, scalar @metadata_order );
-    if ( $info{tmp_extra_name} ) {
-        $info{extra_name} = [];
-        @{ $info{extra_name} } = split /$EXTRA_NAME_SEP/, $info{tmp_extra_name};
-        delete $info{tmp_extra_name};
-    }
-    $R->DEBUG && $R->scrib( 1, "Search run: $info{date}; items saved: $info{num_records};",
-                               "record_class: $info{record_class}" );
-    my $count = 1;
-    my $min = $self->{min} || $p->{min};
-    my $max = $self->{max} || $p->{max};
-    while ( <RESULTS> ) {
-        if ( $min and $count < $min ) { $count++; next; }
-        if ( $max and $count > $max ) { last; }
-        chomp;
-        my ( $item_class, $item_value, @extra ) = split /$RECORD_SEP/, $_, 2 + $info{num_extra};
-        my $result_info = { id => $item_value };
-        $result_info->{class} = $item_class if ( $info{record_class} eq $MIXED_IDENTIFIER );
-
-        # Set the extra information in the result -- note that it's
-        # ALWAYS an arrayref, even if there's only one.
-
-        if ( $info{num_extra} ) {
-            $result_info->{extra} = \@extra;
-        }
-        push @{ $info{result_list} }, $result_info;
-        $count++;
-    }
-    close( RESULTS );
+    $info{num_records} = $count;
     return \%info;
 }
 
 
-sub assign_results_to_object {
-    my ( $self, $result_info ) = @_;
-    $self->{num_records}  = $result_info->{num_records};
-    $self->{date}         = $result_info->{date};
-    $self->{extra_name}   = $result_info->{extra_name};
-    $self->{num_extra}    = $result_info->{num_extra};
-    $self->{record_class} = $result_info->{record_class};
-    $self->{result_list}  = $result_info->{result_list};
-    return $self;
-}
-
-
-sub build_results_filename {
-    my ( $item, $p_search_id ) = @_;
-    my $search_id = ( ref $item and $item->{search_id} ) ? $item->{search_id} : $p_search_id;
-    unless ( $search_id ) {
-        die "Cannot build a results filename without a search_id as property or parameter!\n";
+sub persist_iterator {
+    my ( $self, $out, $to_save, $params ) = @_;
+    my %info = ( record_class => undef, num_records => 0 );
+    while ( my $item = $to_save->get_next ) {
+        $info{record_class} ||= ref $item;
+        $out->print( join( $RECORD_SEP, $info{record_class}, $item->id ), "\n" );
+        $info{num_records}++;
     }
-    return join( '/', OpenInteract::Request->instance->CONFIG->get_dir( 'overflow' ),
-                      $search_id );
-}
-
-
-# Don't save the 'search_id' parameter into the object yet, since
-# this method only ensures that we *can* create a file with the
-# search_id
-
-sub get_persistent_info {
-    my ( $class ) = @_;
-    my $R = OpenInteract::Request->instance;
-    my $overflow_dir = $R->CONFIG->get_dir( 'overflow' );
-    unless ( -d $overflow_dir ) {
-        $R->scrib( 0, "Search results directory ($overflow_dir) is not a directory" );
-        die "Configuration option for writing search results ($overflow_dir) is not a directory";
-    }
-    unless ( -w $overflow_dir ) {
-        $R->scrib( 0, "Search results directory ($overflow_dir) is not writeable" );
-        die "Configuration option for writing search results ($overflow_dir) is not writeable";
-    }
-    while ( 1 ) {
-        my $search_id = SPOPS::Utility->generate_random_code( FILENAME_WIDTH );
-        my $filename = $class->build_results_filename( $search_id );
-        next if ( -f $filename || -f "$filename.$LOCK_EXT" );
-        $R->DEBUG && $R->scrib( 1, "Found non-existent search info: ($search_id) ($filename)" );
-        return ( $search_id, $filename );
-    }
-}
-
-
-# Lock the results file using another file
-
-sub results_lock {
-    my ( $class, $filename ) = @_;
-    open( LOCK, "> $filename.$LOCK_EXT" ) 
-        || die "Cannot open lockfile ($filename.$LOCK_EXT): $!";
-    print LOCK scalar localtime;
-    close( LOCK );
-}
-
-
-# Unlock the results file by deleting the lockfile
-
-sub results_unlock {
-    my ( $class, $filename ) = @_;
-    unlink( "$filename.$LOCK_EXT" ) 
-          || die "Cannot remove lockfile ($filename.$LOCK_EXT): $!";
-}
-
-
-# Clear out the results, including the lockfile. This is called if we
-# encounter some sort of error in the middle of writing.
-
-sub results_clear {
-    my ( $class, $filename ) = @_;
-    $class->results_unlock( $filename );
-    unlink( $filename ); # don't die here b/c if this is called we have bigger problems...
+    return \%info;
 }
 
 
@@ -369,6 +197,356 @@ sub review_results_class {
 }
 
 
+########################################
+# METADATA
+########################################
+
+sub persist_meta {
+    my ( $self, $out, $info, $params ) = @_;
+    $self->{results_dir} ||= $self->find_results_dir;
+    my %meta_info = (
+         time         => time,
+         num_records  => $info->{num_records},
+         record_class => $info->{record_class},
+         has_extra    => $info->{has_extra},
+         extra_name   => $info->{extra_name},
+         num_extra    => ( ref $info->{extra_name} eq 'ARRAY' )
+                           ? scalar $info->{extra_name} : 0,
+         filename     => $params->{filename},
+         results_dir  => $self->{results_dir},
+         search_id    => $params->{search_id},
+    );
+    $out->print( Dumper( \%meta_info ) );
+}
+
+
+sub get_meta {
+    my ( $self, $search_id ) = @_;
+    my $meta_filename = $self->build_meta_filename( $search_id );
+    return {} unless ( -f $meta_filename );
+    eval { open( META, $meta_filename ) || die "Cannot open ($meta_filename): $!" };
+    if ( $@ ) {
+        my $R = OpenInteract::Request->instance;
+        $R->scrib( 0, "Error opening meta file. $@" );
+        return {};
+    }
+    local $/ = undef;
+    no strict 'vars';
+    my $meta_info = eval <META>;
+    close( META );
+    $meta_info->{date} = scalar localtime( $meta_info->{time} );
+    return $meta_info;
+}
+
+
+########################################
+# RETRIEVE RESULTS
+########################################
+
+# Returns either ( \@classes, \@ids, $num_records ) or \@ids depending
+# on context
+
+sub retrieve {
+    my ( $self, $p ) = @_;
+    unless ( $self->{search_id} ) {
+        die "Cannot retrieve results without a search_id! Please ",
+            "set at object initialization or as a property of the ",
+            "object before running retrieve().\n";
+    }
+
+    # 'min' and 'max' can be properties or passed in
+
+    $self->{min} ||= $p->{min};
+    $self->{max} ||= $p->{max};
+
+    # Clear out the number of records
+
+    $self->{num_records} = 0;
+
+    my $R = OpenInteract::Request->instance;
+    $R->DEBUG && $R->scrib( 1, "Retrieving raw search results for ",
+                               "ID ($self->{search_id})" );
+    $self->assign_results_to_object( $self->retrieve_raw_results( $p ) );
+
+    # If they asked for an iterator return it, but first clear out any
+    # min/max values since they've already been preselected
+
+    if ( $p->{return} eq 'iterator' ) {
+        $self->{min} = $self->{max} = $p->{min} = $p->{max} = 0;
+        return $self->retrieve_iterator( $p );
+    }
+    return $self;
+}
+
+
+# Note that this only works on saved SPOPS objects
+
+sub retrieve_iterator {
+    my ( $self, $p ) = @_;
+    require OpenInteract::ResultsIterator;
+
+    # 'min' and 'max' can be properties or passed in
+
+    $self->{min}         ||= $p->{min};
+    $self->{max}         ||= $p->{max};
+
+    my $R = OpenInteract::Request->instance;
+    $R->DEBUG && $R->scrib( 1, "Retrieving search iterator for ID",
+                               "($self->{search_id})" );
+
+    unless ( $self->{result_list} ) {
+        $self->assign_results_to_object( $self->retrieve_raw_results( $p ) );
+    }
+    if ( $self->{num_records} <= 0 ) {
+        return OpenInteract::ResultsIterator->new({ results => $self });
+    }
+
+    unless ( $self->{record_class} ) {
+        die "Cannot create iterator! Search results were not saved with a classname.",
+            "(Search ID: $self->{search_id})\n";
+    }
+    if ( $self->{record_class} eq $MIXED_IDENTIFIER ) {
+        $self->{min} = $self->{max} = 0;
+        return OpenInteract::ResultsIterator->new({
+                     results       => $self,
+                     skip_security => $p->{skip_security} });
+    }
+    return $self->{record_class}->fetch_iterator({ id_list => $self->get_id_list });
+}
+
+
+# Take the result list internally and return a list of all IDs
+
+sub get_id_list {
+    my ( $self ) = @_;
+    unless ( $self->{result_list} ) {
+        die "Results not yet saved for this object!\n"
+    }
+    return [ map { $_->{id} } @{ $self->{result_list} } ];
+}
+
+
+
+# Returns a hashref of information about the results, including the ID
+# list, the class, results save time. Pass in min/max in the hashref
+# (second arg) to have the results be paged.
+
+sub retrieve_raw_results {
+    my ( $self, $p ) = @_;
+    my $R = OpenInteract::Request->instance;
+    die "No search_id defined in object!\n" unless ( $self->{search_id} );
+
+    my $meta_info = $self->get_meta({ search_id => $self->{search_id} });
+    $R->DEBUG && $R->scrib( 1, "Run on: ($meta_info->{date})",
+                               "Saved: ($meta_info->{num_records})",
+                               "Type: ($meta_info->{record_class})" );
+    if ( $meta_info->{num_records} <= 0 ) {
+        return $meta_info;
+    }
+
+    my $filename = $self->build_results_filename;
+    eval { open( RESULTS, $filename ) || die "Cannot open ($filename): $!\n" };
+    if ( $@ ) {
+        $R->scrib( 0, "Search result retrieval failure. $@" );
+        return undef;
+    }
+
+    my $count = 1;
+    my $min = $self->{min} || $p->{min};
+    my $max = $self->{max} || $p->{max};
+    while ( <RESULTS> ) {
+        if ( $min and $count < $min ) { $count++; next; }
+        if ( $max and $count > $max ) { last; }
+        chomp;
+        my ( $item_class, $item_value, @extra ) = split /$RECORD_SEP/, $_, 2 + $meta_info->{num_extra};
+        my $result_info = { id => $item_value };
+        if ( $meta_info->{record_class} eq $MIXED_IDENTIFIER ) {
+            $result_info->{class} = $item_class;
+        }
+
+        # Set the extra information in the result -- note that it's
+        # ALWAYS an arrayref, even if there's only one.
+
+        $result_info->{extra} = \@extra if ( $meta_info->{num_extra} );
+        push @{ $meta_info->{result_list} }, $result_info;
+        $count++;
+    }
+    close( RESULTS );
+    return $meta_info;
+}
+
+
+sub assign_results_to_object {
+    my ( $self, $result_info ) = @_;
+    for ( qw( num_records time date extra_name num_extra record_class result_list ) ) {
+        $self->{ $_ } = $result_info->{ $_ };
+    }
+    return $self;
+}
+
+
+########################################
+# PAGING METHODS
+########################################
+
+
+# Note that $item can be a class or an object -- if object, we'll take
+# the number of records from it.
+
+sub find_total_page_count {
+    my ( $item, $per_page, $p_num_records ) = @_;
+    my $num_records = ( ref $item and $item->{num_records} )
+                        ? $item->{num_records} : $p_num_records;
+    return 0 unless ( $per_page and $num_records );
+    my $num_pages = $num_records / $per_page;
+    return ( int $num_pages != $num_pages ) ? int( $num_pages ) + 1 : int $num_pages;
+}
+
+
+sub find_page_boundaries {
+    my ( $class, $page_num, $per_page ) = @_;
+    return ( 0, 0 ) unless ( $page_num and $per_page );
+    my $max = $page_num * $per_page;
+    my $min = $max - $per_page;
+    return ( $min, $max );
+}
+
+
+
+########################################
+# FILENAME/DIRECTORY METHODS
+########################################
+
+sub get_results_dir {
+    my ( $class ) = @_;
+    my $R = OpenInteract::Request->instance;
+    return $R->CONFIG->get_dir( 'overflow' );
+}
+
+
+sub build_results_filename {
+    my ( $self, $p_search_id ) = @_;
+    my $search_id = $self->{search_id} || $p_search_id;
+    unless ( $search_id ) {
+        die "Cannot build a results filename without a search_id ",
+            "as property or parameter!\n";
+    }
+    $self->{results_dir} ||= $self->get_results_dir;
+    return join( '/', $self->{results_dir}, $search_id );
+}
+
+
+sub build_meta_filename {
+    my ( $self, $p_search_id ) = @_;
+    my $search_id = $self->{search_id} || $p_search_id;
+    unless ( $search_id ) {
+        die "Cannot build a results filename without a search_id ",
+            "as property or parameter!\n";
+    }
+    $self->{results_dir} ||= $self->get_results_dir;
+    return join( '/', $self->{results_dir}, "$search_id.meta" );
+}
+
+
+sub build_lock_filename {
+    my ( $self, $p_search_id ) = @_;
+    my $search_id = $self->{search_id} || $p_search_id;
+    unless ( $search_id ) {
+        die "Cannot build a lock filename without a search_id ",
+            "as property or parameter!\n";
+    }
+    $self->{results_dir} ||= $self->get_results_dir;
+    return join( '/', $self->{results_dir}, "$search_id.$LOCK_EXT" );
+}
+
+
+sub get_all_result_filenames {
+    my ( $self ) = @_;
+    $self->{results_dir} ||= $self->get_results_dir;
+    opendir( RESULTS, $self->{results_dir} )
+             || die "Cannot open results directory ($self->{results_dir}): $!";
+    my @results_files = grep ! /\./, grep { length $_ == FILENAME_WIDTH } readdir( RESULTS );
+    closedir( RESULTS );
+    return \@results_files;
+}
+
+
+# Clear out the results, including the lockfile. This is called if we
+# encounter some sort of error in the middle of writing. Don't die
+# during this method because if this is called we have bigger
+# problems...
+
+sub results_clear {
+    my ( $self, $search_id ) = @_;
+    eval { $self->results_unlock( $search_id ) };
+    unlink( $self->build_results_filename( $search_id ) );
+    unlink( $self->build_meta_filename( $search_id ) );
+}
+
+
+########################################
+# SEARCH ID
+########################################
+
+# Don't save the 'search_id' parameter into the object yet, since
+# this method only ensures that we *can* create a file with the
+# search_id
+
+sub generate_search_id {
+    my ( $self ) = @_;
+    my $R = OpenInteract::Request->instance;
+    $self->{results_dir} ||= $self->get_results_dir;
+
+    unless ( -d $self->{results_dir} ) {
+        $R->scrib( 0, "Search results directory ($self->{results_dir})",
+                      "is not a directory" );
+        die "Configuration option for writing search results ",
+            "($self->{results_dir}) is not a directory";
+    }
+    unless ( -w $self->{results_dir} ) {
+        $R->scrib( 0, "Search results dir ($self->{results_dir}) is not writeable" );
+        die "Configuration option for writing search results ",
+            "($self->{results_dir}) exists but is not writeable";
+    }
+    while ( 1 ) {
+        my $search_id = SPOPS::Utility->generate_random_code( FILENAME_WIDTH );
+        my $filename = $self->build_results_filename( $search_id );
+        my $lockfile = $self->build_lock_filename( $search_id );
+        next if ( -f $filename || -f $lockfile );
+        $R->DEBUG && $R->scrib( 1, "Found non-existent search info for ",
+                                   "($search_id) in ($self->{results_dir})" );
+        return $search_id;
+    }
+}
+
+
+########################################
+# LOCKING
+########################################
+
+# Lock the results file using another file
+
+sub results_lock {
+    my ( $self, $search_id ) = @_;
+    my $lock_file = $self->build_lock_filename( $search_id );
+    open( LOCK, "> $lock_file" )
+        || die "Cannot open lockfile ($lock_file): $!";
+    print LOCK scalar localtime;
+    close( LOCK );
+}
+
+
+# Unlock the results file by deleting the lockfile.
+
+sub results_unlock {
+    my ( $self, $search_id ) = @_;
+    my $lock_file = $self->build_lock_filename( $search_id );
+    return unless ( -f $lock_file );
+    unlink( $lock_file )
+          || die "Cannot remove lockfile ($lock_file): $!";
+}
+
+
 1;
 
 __END__
@@ -389,33 +567,33 @@ OpenInteract::ResultsManage - Save and retrieve generic search results
 
  my $results = OpenInteract::ResultsManage->new();
  $results->save( \@id_list );
- $R->{session}->{this_search_id} = $results->{search_id};
+ $R->{session}{this_search_id} = $results->{search_id};
 
  ... another request from this user ...
 
- my $results = OpenInteract::ResultsManage->new({ 
-                              search_id => $R->{session}->{this_search_id} });
+ my $results = OpenInteract::ResultsManage->new({
+                              search_id => $R->{session}{this_search_id} });
  my $result_list = $results->retrieve();
 
  # Use with paged results
 
- my $results = OpenInteract::ResultsManage->new(); 
+ my $results = OpenInteract::ResultsManage->new();
  $results->save( \@id_list );
- $R->{session}->{this_search_id} = $results->{search_id};
+ $R->{session}{this_search_id} = $results->{search_id};
  my $page_num = $R->apache->param( 'pagenum' );
  my ( $min, $max ) = $results->find_page_boundaries( $page_num, $HITS_PER_PAGE );
  my ( $results, $total_count ) = $results->retrieve({ min => $min, max => $max } );
  my $total_pages = $results->find_total_page_count( $HITS_PER_PAGE );
  my $total_hits = $results->{num_records};
- 
- # Can now print "Page $page_num of $total_pages" or you 
- # can pass this information to the template and use the 
+
+ # Can now print "Page $page_num of $total_pages" or you
+ # can pass this information to the template and use the
  # 'page_count' component and pass it 'total_pages',
  # 'current_pagenum', and a 'url' to get back to this page:
 
- [%- comp( 'page_count', total_pages     = 5,
-                         current_pagenum = 3,
-                         url             = url ) -%]
+ [%- OI.comp( 'page_count', total_pages     = 5,
+                            current_pagenum = 3,
+                            url             = url ) -%]
 
  Displays:
 
@@ -435,17 +613,20 @@ determination.
 The following methods are public and available for OpenInteract
 application developers.
 
-B<save( \@results_list, \%params )>
+B<save( $stuff_to_save, \%params )>
 
-Saves a list of things to be retrieved later. The things can be either
-ID values (simple scalars) or SPOPS objects. If objects are passed in,
-we call C<-E<gt>id()> on each to get the ID value to save.
+Saves a list of things to be retrieved later. The C<$stuff_to_save>
+can be an arrayref of ID values (simple scalars), an arrayref of SPOPS
+objects, or an L<SPOPS::Iterator|SPOPS::Iterator> implementation all
+primed and ready to go. If objects are passed in via a list or an
+iterator, we call C<-E<gt>id()> on each to get the ID value to save.
 
 If objects are used, we also query each one for its class and save
 that information in the search results. Whether you have a homogenous
 resultset or not affects the return values. If it is a homogenous
 resultset we note the class for all objects in the search results
-header. This enables us to create an iterator from the results if
+metadata, which is saved in a separate file from the results
+themselves. This enables us to create an iterator from the results if
 needed.
 
 Parameters:
@@ -458,17 +639,23 @@ B<class> ($) (optional)
 
 You can force all the IDs passed in to be of a particular class.
 
+=item *
+
 B<force_mixed> (bool) (optional)
 
 Forces the resultset to be treated as heterogeneous (mixed) even if
 all objects are of the same class.
 
+=item *
+
 B<extra> (\@) (optional)
 
 Each item represents extra information to save along with each
 result. Each item must be either a scalar (which saves one extra item)
 or an arrayref (which saves a number of extra items).
 
+=item *
+
 B<extra_name> (\@)  (optional)
 
 If you specify extra information you need to give each one a name.
@@ -479,6 +666,9 @@ Returns: an ID you can use to retrieve the search results using
 the C<retrieve()> or C<retrieve_iterator()> methods. If
 you misplace the ID, you cannot get the search results back.
 
+Side effects: the ID returned is also saved in the 'search_id' key of
+the object itself.
+
 Example:
 
  my $results = OpenInteract::ResultsManage->new();
@@ -490,9 +680,8 @@ Example:
 The following parameters are set in the object after a successful
 results save:
 
-'search_id'
-
-'num_records'
+ search_id
+ num_records
 
 Returns: the ID of the search just saved.
 
@@ -580,7 +769,7 @@ B<find_page_boundaries( $page_number, $records_per_page )>
 Returns: An array with the floor and ceiling values to display the
 given page with $records_per_page on the page.
 
-Example: 
+Example:
 
  my ( $min, $max ) = $class->find_page_boundaries( 3, 75 );
  # $min is 226, $max is 300
@@ -592,7 +781,7 @@ Example:
 
 B<build_results_filename()>
 
-B<get_persistent_info()>
+B<generate_search_id()>
 
 B<results_lock()>
 
@@ -703,7 +892,7 @@ B<Objectify?>
 Think about creating a 'search_results' object that can access the
 resultset along with metadata about the results (number of items, time
 searched, etc.). This would likely prove easier to work with in the
-future. 
+future.
 
 What would also be interesting is combine this with the interface for
 L<SPOPS::Iterator> and the currently impelemented
